@@ -19,8 +19,6 @@ Leverages websockify.py by Joel Martin
 '''
 
 import copy
-from http import HTTPStatus
-import os
 import socket
 import sys
 
@@ -67,15 +65,8 @@ class TenantSock(object):
             new_frames, closed = self.reqhandler.recv_frames()
             # flatten frames onto queue
             for frame in new_frames:
-                # The socket returns (byte) strings in Python 2...
-                if six.PY2:
-                    self.queue.extend(frame)
-                # ...and integers in Python 3. For the Python 3 case, we need
-                # to convert these to characters using 'chr' and then, as this
-                # returns unicode, convert the result to byte strings.
-                else:
-                    self.queue.extend(
-                        [six.binary_type(chr(c), 'ascii') for c in frame])
+                self.queue.extend(
+                    [six.binary_type(chr(c), 'ascii') for c in frame])
 
             if closed:
                 break
@@ -290,22 +281,6 @@ class NovaProxyRequestHandler(websockify.ProxyRequestHandler):
     def socket(self, *args, **kwargs):
         return websockifyserver.WebSockifyServer.socket(*args, **kwargs)
 
-    def send_head(self):
-        # This code is copied from this example patch:
-        # https://bugs.python.org/issue32084#msg306545
-        path = self.translate_path(self.path)
-        if os.path.isdir(path):
-            parts = urlparse.urlsplit(self.path)
-            if not parts.path.endswith('/'):
-                # Browsers interpret "Location: //uri" as an absolute URI
-                # like "http://URI"
-                if self.path.startswith('//'):
-                    self.send_error(HTTPStatus.BAD_REQUEST,
-                                    "URI must not start with //")
-                    return None
-
-        return super(NovaProxyRequestHandler, self).send_head()
-
 
 class NovaWebSocketProxy(websockify.WebSocketProxy):
     def __init__(self, *args, **kwargs):
