@@ -17,6 +17,7 @@
 
 import errno
 import os
+import shutil
 
 from oslo_utils import fileutils
 
@@ -34,7 +35,7 @@ def readfile(path):
 
 @nova.privsep.sys_admin_pctxt.entrypoint
 def writefile(path, mode, content):
-    if not os.path.exists(path):
+    if not os.path.exists(os.path.dirname(path)):
         raise exception.FileNotFound(file_path=path)
     with open(path, mode) as f:
         f.write(content)
@@ -48,10 +49,21 @@ def readlink(path):
 
 
 @nova.privsep.sys_admin_pctxt.entrypoint
-def chown(path, uid=-1, gid=-1):
+def chown(
+    path: str, uid: int = -1, gid: int = -1, recursive: bool = False,
+) -> None:
     if not os.path.exists(path):
         raise exception.FileNotFound(file_path=path)
-    return os.chown(path, uid, gid)
+
+    if not recursive or os.path.isfile(path):
+        return os.chown(path, uid, gid)
+
+    for root, dirs, files in os.walk(path):
+        os.chown(root, uid, gid)
+        for item in dirs:
+            os.chown(os.path.join(root, item), uid, gid)
+        for item in files:
+            os.chown(os.path.join(root, item), uid, gid)
 
 
 @nova.privsep.sys_admin_pctxt.entrypoint
@@ -66,6 +78,11 @@ def chmod(path, mode):
     os.chmod(path, mode)
 
 
+@nova.privsep.sys_admin_pctxt.entrypoint
+def move_tree(source_path: str, dest_path: str) -> None:
+    shutil.move(source_path, dest_path)
+
+
 @nova.privsep.sys_admin_pctxt.entrypoint
 def utime(path):
     if not os.path.exists(path):
@@ -94,32 +111,24 @@ class path(object):
 
 @nova.privsep.sys_admin_pctxt.entrypoint
 def last_bytes(path, num):
-    # NOTE(mikal): this is implemented in this contrived manner because you
-    # can't mock a decorator in python (they're loaded at file parse time,
-    # and the mock happens later).
-    with open(path, 'rb') as f:
-        return _last_bytes_inner(f, num)
-
-
-def _last_bytes_inner(file_like_object, num):
     """Return num bytes from the end of the file, and remaining byte count.
 
-    :param file_like_object: The file to read
+    :param path: The file to read
     :param num: The number of bytes to return
 
     :returns: (data, remaining)
     """
+    with open(path, 'rb') as f:
+        try:
+            f.seek(-num, os.SEEK_END)
+        except IOError as e:
+            # seek() fails with EINVAL when trying to go before the start of
+            # the file. It means that num is larger than the file size, so
+            # just go to the start.
+            if e.errno == errno.EINVAL:
+                f.seek(0, os.SEEK_SET)
+            else:
+                raise
 
-    try:
-        file_like_object.seek(-num, os.SEEK_END)
-    except IOError as e:
-        # seek() fails with EINVAL when trying to go before the start of
-        # the file. It means that num is larger than the file size, so
-        # just go to the start.
-        if e.errno == errno.EINVAL:
-            file_like_object.seek(0, os.SEEK_SET)
-        else:
-            raise
-
-    remaining = file_like_object.tell()
-    return (file_like_object.read(), remaining)
+        remaining = f.tell()
+        return (f.read(), remaining)
