@@ -33,7 +33,6 @@ class TestLiveMigrationWithoutMultiplePortBindings(
     """
 
     ADMIN_API = True
-    api_major_version = 'v2.1'
     microversion = 'latest'
 
     def list_extensions(self, *args, **kwargs):
@@ -52,7 +51,6 @@ class TestLiveMigrationWithoutMultiplePortBindings(
         }
 
     def setUp(self):
-        self.flags(instances_path=self.useFixture(fixtures.TempDir()).path)
         super().setUp()
         self.neutron.list_extensions = self.list_extensions
         self.neutron_api = neutron.API()
@@ -63,13 +61,16 @@ class TestLiveMigrationWithoutMultiplePortBindings(
             'nova.virt.libvirt.driver.connector',
             fake_os_brick_connector))
 
-        self.start_computes({
-            'start_host': fakelibvirt.HostInfo(
+        self.start_compute(
+            hostname='start_host',
+            host_info=fakelibvirt.HostInfo(
                 cpu_nodes=1, cpu_sockets=1, cpu_cores=4, cpu_threads=2,
-                kB_mem=10740000),
-            'end_host': fakelibvirt.HostInfo(
+                kB_mem=10740000))
+        self.start_compute(
+            hostname='end_host',
+            host_info=fakelibvirt.HostInfo(
                 cpu_nodes=1, cpu_sockets=1, cpu_cores=4, cpu_threads=2,
-                kB_mem=10740000)})
+                kB_mem=10740000))
 
         self.ctxt = context.get_admin_context()
         # TODO(sean-k-mooney): remove this when it is part of ServersTestBase
