@@ -174,16 +174,26 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
                          set([1, 2]))
 
     def test_support_requests_numa(self):
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0),
-                 objects.InstanceNUMACell(id=1, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+            objects.InstanceNUMACell(
+                id=1, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         self.assertTrue(self.pci_stats.support_requests(pci_requests, cells))
 
     def test_support_requests_numa_failed(self):
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         self.assertFalse(self.pci_stats.support_requests(pci_requests, cells))
 
     def test_support_requests_no_numa_info(self):
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         pci_requests = self._get_fake_requests(vendor_ids=['v3'])
         self.assertTrue(self.pci_stats.support_requests(pci_requests, cells))
 
@@ -197,7 +207,10 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
         # numa node 1 has 1 device with vendor_id 'v2'
         # we request two devices with vendor_id 'v1' and 'v2'.
         # pci_numa_policy is 'preferred' so we can ignore numa affinity
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         pci_requests = self._get_fake_requests(
             numa_policy=fields.PCINUMAAffinityPolicy.PREFERRED)
 
@@ -206,7 +219,10 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
     def test_support_requests_no_numa_info_pci_numa_policy_required(self):
         # pci device with vendor_id 'v3' has numa_node=None.
         # pci_numa_policy is 'required' so we can't use this device
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         pci_requests = self._get_fake_requests(vendor_ids=['v3'],
             numa_policy=fields.PCINUMAAffinityPolicy.REQUIRED)
 
@@ -227,21 +243,31 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
                           pci_requests_multiple))
 
     def test_consume_requests_numa(self):
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0),
-                 objects.InstanceNUMACell(id=1, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+            objects.InstanceNUMACell(
+                id=1, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         devs = self.pci_stats.consume_requests(pci_requests, cells)
         self.assertEqual(2, len(devs))
         self.assertEqual(set(['v1', 'v2']),
                          set([dev.vendor_id for dev in devs]))
 
     def test_consume_requests_numa_failed(self):
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         self.assertIsNone(self.pci_stats.consume_requests(pci_requests, cells))
 
     def test_consume_requests_no_numa_info(self):
-        cells = [objects.InstanceNUMACell(id=0, cpuset=set(), memory=0)]
+        cells = [
+            objects.InstanceNUMACell(
+                id=0, cpuset=set(), pcpuset=set(), memory=0),
+        ]
         pci_request = [objects.InstancePCIRequest(count=1,
-                    spec=[{'vendor_id': 'v3'}])]
+                                                  spec=[{'vendor_id': 'v3'}])]
         devs = self.pci_stats.consume_requests(pci_request, cells)
         self.assertEqual(1, len(devs))
         self.assertEqual(set(['v3']),
@@ -258,8 +284,10 @@ class PciDeviceStatsTestCase(test.NoDBTestCase):
         ``expected``.
         """
         self._add_fake_devs_with_numa()
-        cells = [objects.InstanceNUMACell(id=id, cpuset=set(), memory=0)
-                 for id in cell_ids]
+        cells = [
+            objects.InstanceNUMACell(
+                id=id, cpuset=set(), pcpuset=set(), memory=0)
+            for id in cell_ids]
 
         pci_requests = self._get_fake_requests(vendor_ids=[vendor_id],
             numa_policy=policy, count=count)
@@ -533,30 +561,6 @@ class PciDeviceStatsWithTagsTestCase(test.NoDBTestCase):
         self.pci_stats.remove_device(dev2)
         self._assertPools()
 
-    def test_update_device(self):
-        # Update device type of one of the device from type-PCI to
-        # type-PF. Verify if the existing pool is updated and a new
-        # pool is created with dev_type type-PF.
-        self._create_pci_devices()
-        dev1 = self.pci_tagged_devices.pop()
-        dev1.dev_type = 'type-PF'
-        self.pci_stats.update_device(dev1)
-        self.assertEqual(3, len(self.pci_stats.pools))
-        self._assertPoolContent(self.pci_stats.pools[0], '1137', '0072',
-                                len(self.pci_untagged_devices))
-        self.assertEqual(self.pci_untagged_devices,
-                         self.pci_stats.pools[0]['devices'])
-        self._assertPoolContent(self.pci_stats.pools[1], '1137', '0071',
-                                len(self.pci_tagged_devices),
-                                physical_network='physnet1')
-        self.assertEqual(self.pci_tagged_devices,
-                         self.pci_stats.pools[1]['devices'])
-        self._assertPoolContent(self.pci_stats.pools[2], '1137', '0071',
-                                1,
-                                physical_network='physnet1')
-        self.assertEqual(dev1,
-                         self.pci_stats.pools[2]['devices'][0])
-
 
 class PciDeviceVFPFStatsTestCase(test.NoDBTestCase):
 
