@@ -20,7 +20,6 @@ from nova.tests.unit import matchers
 from nova.tests.unit.virt.libvirt import fake_libvirt_data
 from nova.virt.libvirt import config
 from nova.virt.libvirt import designer
-from nova.virt.libvirt import host
 
 
 class DesignerTestCase(test.NoDBTestCase):
@@ -61,24 +60,11 @@ class DesignerTestCase(test.NoDBTestCase):
 
     def test_set_vif_host_backend_ethernet_config_libvirt_1_3_3(self):
         conf = config.LibvirtConfigGuestInterface()
-        mock_host = mock.Mock(autospec=host.Host)
-        mock_host.has_min_version.return_value = True
-        designer.set_vif_host_backend_ethernet_config(
-            conf, 'fake-tap', mock_host)
+        designer.set_vif_host_backend_ethernet_config(conf, 'fake-tap')
         self.assertEqual('ethernet', conf.net_type)
         self.assertEqual('fake-tap', conf.target_dev)
         self.assertIsNone(conf.script)
 
-    def test_set_vif_host_backend_ethernet_config_libvirt_pre_1_3_3(self):
-        conf = config.LibvirtConfigGuestInterface()
-        mock_host = mock.Mock(autospec=host.Host)
-        mock_host.has_min_version.return_value = False
-        designer.set_vif_host_backend_ethernet_config(
-            conf, 'fake-tap', mock_host)
-        self.assertEqual('ethernet', conf.net_type)
-        self.assertEqual('fake-tap', conf.target_dev)
-        self.assertEqual('', conf.script)
-
     def test_set_vif_host_backend_802qbg_config(self):
         conf = config.LibvirtConfigGuestInterface()
         designer.set_vif_host_backend_802qbg_config(conf, 'fake-devname',
