@@ -22,7 +22,6 @@ from os_win import utilsfactory
 from oslo_log import log as logging
 
 from nova.compute import task_states
-from nova.i18n import _
 from nova.image import glance
 from nova.virt.hyperv import pathutils
 
@@ -38,8 +37,7 @@ class SnapshotOps(object):
     def _save_glance_image(self, context, image_id, image_vhd_path):
         (glance_image_service,
          image_id) = glance.get_remote_image_service(context, image_id)
-        image_metadata = {"is_public": False,
-                          "disk_format": "vhd",
+        image_metadata = {"disk_format": "vhd",
                           "container_format": "bare"}
         with self._pathutils.open(image_vhd_path, 'rb') as f:
             glance_image_service.update(context, image_id, image_metadata, f,
@@ -112,7 +110,7 @@ class SnapshotOps(object):
                 LOG.debug("Removing snapshot %s", image_id)
                 self._vmutils.remove_vm_snapshot(snapshot_path)
             except Exception:
-                LOG.exception(_('Failed to remove snapshot for VM %s'),
+                LOG.exception('Failed to remove snapshot for VM %s',
                               instance_name, instance=instance)
             if export_dir:
                 LOG.debug('Removing directory: %s', export_dir)
