@@ -35,7 +35,6 @@ from oslo_service import loopingcall
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
-import six
 
 from nova.compute import power_state
 from nova import exception
@@ -74,6 +73,25 @@ LIBVIRT_POWER_STATE = {
     VIR_DOMAIN_PMSUSPENDED: power_state.SUSPENDED,
 }
 
+# https://libvirt.org/html/libvirt-libvirt-domain.html#virDomainBlockJobType
+VIR_DOMAIN_BLOCK_JOB_TYPE_UNKNOWN = 0
+VIR_DOMAIN_BLOCK_JOB_TYPE_PULL = 1
+VIR_DOMAIN_BLOCK_JOB_TYPE_COPY = 2
+VIR_DOMAIN_BLOCK_JOB_TYPE_COMMIT = 3
+VIR_DOMAIN_BLOCK_JOB_TYPE_ACTIVE_COMMIT = 4
+VIR_DOMAIN_BLOCK_JOB_TYPE_BACKUP = 5
+VIR_DOMAIN_BLOCK_JOB_TYPE_LAST = 6
+
+LIBVIRT_BLOCK_JOB_TYPE = {
+    VIR_DOMAIN_BLOCK_JOB_TYPE_UNKNOWN: 'UNKNOWN',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_PULL: 'PULL',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_COPY: 'COPY',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_COMMIT: 'COMMIT',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_ACTIVE_COMMIT: 'ACTIVE_COMMIT',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_BACKUP: 'BACKUP',
+    VIR_DOMAIN_BLOCK_JOB_TYPE_LAST: 'LAST',
+}
+
 
 class Guest(object):
 
@@ -118,7 +136,7 @@ class Guest(object):
         :returns guest.Guest: Guest ready to be launched
         """
         try:
-            if six.PY3 and isinstance(xml, six.binary_type):
+            if isinstance(xml, bytes):
                 xml = xml.decode('utf-8')
             guest = host.write_instance_config(xml)
         except Exception:
@@ -211,31 +229,25 @@ class Guest(object):
         return interfaces
 
     def get_interface_by_cfg(self, cfg):
-        """Lookup a full LibvirtConfigGuestInterface with
-        LibvirtConfigGuestInterface generated
+        """Lookup a full LibvirtConfigGuestDevice with
+        LibvirtConfigGuesDevice generated
         by nova.virt.libvirt.vif.get_config.
 
         :param cfg: config object that represents the guest interface.
-        :type cfg: LibvirtConfigGuestInterface object
-        :returns: nova.virt.libvirt.config.LibvirtConfigGuestInterface instance
+        :type cfg: a subtype of LibvirtConfigGuestDevice object
+        :returns: nova.virt.libvirt.config.LibvirtConfigGuestDevice instance
             if found, else None
         """
 
         if cfg:
-            interfaces = self.get_all_devices(
-                vconfig.LibvirtConfigGuestInterface)
+            interfaces = self.get_all_devices(type(cfg))
             for interface in interfaces:
-                # NOTE(leehom) LibvirtConfigGuestInterface get from domain and
-                # LibvirtConfigGuestInterface generated by
+                # NOTE(leehom) LibvirtConfigGuest get from domain and
+                # LibvirtConfigGuest generated by
                 # nova.virt.libvirt.vif.get_config must be identical.
-                # NOTE(arches) Skip checking target_dev for vhostuser
-                # vif type; target_dev is not a valid value for vhostuser.
-                if (interface.mac_addr == cfg.mac_addr and
-                        interface.net_type == cfg.net_type and
-                        interface.source_dev == cfg.source_dev and
-                        (cfg.net_type == 'vhostuser' or
-                            interface.target_dev == cfg.target_dev) and
-                        interface.vhostuser_path == cfg.vhostuser_path):
+                # NOTE(gibi): LibvirtConfigGuest subtypes does a custom
+                # equality check based on available information on nova side
+                if cfg == interface:
                     return interface
 
     def get_vcpus_info(self):
@@ -286,7 +298,7 @@ class Guest(object):
         flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
 
         device_xml = conf.to_xml()
-        if six.PY3 and isinstance(device_xml, six.binary_type):
+        if isinstance(device_xml, bytes):
             device_xml = device_xml.decode('utf-8')
 
         LOG.debug("attach device xml: %s", device_xml)
@@ -355,8 +367,8 @@ class Guest(object):
         return devs
 
     def detach_device_with_retry(self, get_device_conf_func, device, live,
-                                 max_retry_count=7, inc_sleep_time=10,
-                                 max_sleep_time=60,
+                                 max_retry_count=7, inc_sleep_time=2,
+                                 max_sleep_time=30,
                                  alternative_device_name=None,
                                  supports_device_missing_error_code=False):
         """Detaches a device from the guest. After the initial detach request,
@@ -486,7 +498,7 @@ class Guest(object):
         flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
 
         device_xml = conf.to_xml()
-        if six.PY3 and isinstance(device_xml, six.binary_type):
+        if isinstance(device_xml, bytes):
             device_xml = device_xml.decode('utf-8')
 
         LOG.debug("detach device xml: %s", device_xml)
@@ -590,7 +602,7 @@ class Guest(object):
         flags |= quiesce and libvirt.VIR_DOMAIN_SNAPSHOT_CREATE_QUIESCE or 0
 
         device_xml = conf.to_xml()
-        if six.PY3 and isinstance(device_xml, six.binary_type):
+        if isinstance(device_xml, bytes):
             device_xml = device_xml.decode('utf-8')
 
         self._domain.snapshotCreateXML(device_xml, flags=flags)
@@ -666,13 +678,6 @@ class Guest(object):
                 not params.get('migrate_disks')):
             flags &= ~libvirt.VIR_MIGRATE_NON_SHARED_INC
 
-        # In the Python2 libvirt bindings, strings passed to
-        # migrateToURI3 via params must not be unicode.
-        if six.PY2:
-            params = {key: encodeutils.to_utf8(value)
-                      if isinstance(value, six.text_type) else value
-                      for key, value in params.items()}
-
         self._domain.migrateToURI3(
             destination, params=params, flags=flags)
 
@@ -863,17 +868,16 @@ class BlockDevice(object):
         if status is None:
             return True
 
-        # NOTE(slaweq): because of bug in libvirt, which is described in
-        # http://www.redhat.com/archives/libvir-list/2016-September/msg00017.html
-        # if status.end == 0 job is not started yet so it is not finished
-        # NOTE(mdbooth): The fix was committed upstream here:
-        #   http://libvirt.org/git/?p=libvirt.git;a=commit;h=988218c
-        # The earliest tag which contains this commit is v2.3.0-rc1, so we
-        # should be able to remove this workaround when MIN_LIBVIRT_VERSION
-        # reaches 2.3.0, or we move to handling job events instead.
+        # Track blockjob progress in DEBUG, helpful when reviewing failures.
+        job_type = LIBVIRT_BLOCK_JOB_TYPE.get(
+            status.job, f"Unknown to Nova ({status.job})")
+        LOG.debug("%(job_type)s block job progress, current cursor: %(cur)s "
+                  "final cursor: %(end)s",
+                  {'job_type': job_type, 'cur': status.cur, 'end': status.end})
+
         # NOTE(lyarwood): Use the mirror element to determine if we can pivot
         # to the new disk once blockjobinfo reports progress as complete.
-        if status.end != 0 and status.cur == status.end:
+        if status.cur == status.end:
             disk = self._guest.get_disk(self._disk)
             if disk and disk.mirror:
                 return disk.mirror.ready == 'yes'
